from ..utils import queue_system
from ..variables import global_vars
import bpy
from bpy.app.handlers import persistent
from .. operators.dont_register import render_operations


def create_timer(the_function,arguments):
    '''avoid adding a function twice.'''

    temp_dic = queue_system.get_queued_items()
    for item in temp_dic:
        func = item[0]
        if func.__code__.co_code == the_function.__code__.co_code:
            print(func.__name__,'already in queue')
            return
        
    queue_system.add_function_to_queue([the_function,arguments])
    queue_system.register_timer()


@persistent
def post_render_job(scene):    
    print('creating timer in post render job')
    create_timer(render_operations.render_post,scene.name)

@persistent
def render_job_cancelled(scene):    
    global_vars.cancelled = True
    print('creating timer in cancel')
    create_timer(render_operations.render_post,scene.name)



def register_handler():
    if post_render_job not in bpy.app.handlers.render_complete:
        bpy.app.handlers.render_complete.append(post_render_job)
    if render_job_cancelled not in bpy.app.handlers.render_cancel:
        bpy.app.handlers.render_cancel.append(render_job_cancelled)

def unregister_handler():
    
    if post_render_job in bpy.app.handlers.render_complete:
        bpy.app.handlers.render_complete.remove(post_render_job)

    if render_job_cancelled in bpy.app.handlers.render_cancel:
        bpy.app.handlers.render_cancel.remove(render_job_cancelled)
            
