
import bpy,random
from ...variables import global_vars

def store_user_render_settings(scene_name):
    '''stores the user's render settings before rendering'''
    if bpy.data.scenes[scene_name].render.engine == 'CYCLES':
        if hasattr( bpy.data.scenes[scene_name].cycles,'time_limit'):
            tl = bpy.data.scenes[scene_name].cycles.time_limit
        else:
            tl = 0
        
        User_Render_Settings = {  
                        'Max_Samples':bpy.data.scenes[scene_name].cycles.samples,
                        'Min_Samples' : bpy.data.scenes[scene_name].cycles.adaptive_min_samples, 
                        'Noise_Threshold' : bpy.data.scenes[scene_name].cycles.adaptive_threshold,
                        'Time_Limit' : tl, 
                        'Use_Adaptive' :bpy.data.scenes[scene_name].cycles.use_adaptive_sampling,                        
                        'Scene_Name' : scene_name
                    }
       
    if User_Render_Settings:
        return User_Render_Settings


def set_random_sample_settings():
    
    bpy.data.scenes['Scene'].cycles.samples = random.randint(12,32)
    bpy.data.scenes['Scene'].cycles.adaptive_min_samples = random.randint(0,8)
    bpy.data.scenes['Scene'].cycles.adaptive_threshold = round(random.uniform(0.1,0.5),4)
    bpy.data.scenes['Scene'].cycles.use_adaptive_sampling = random.choice([True, False])


def revert_all_scenes_back_to_the_users_original_render_settings():
    if global_vars.all_user_render_settings is not None:
        for settings in global_vars.all_user_render_settings:
            bpy.data.scenes[settings['Scene_Name']].cycles.samples = settings['Max_Samples']
            bpy.data.scenes[settings['Scene_Name']].cycles.adaptive_min_samples = settings['Min_Samples']
            bpy.data.scenes[settings['Scene_Name']].cycles.adaptive_threshold = settings['Noise_Threshold']
            if hasattr(bpy.data.scenes[settings['Scene_Name']].cycles,'time_limit'):
                bpy.data.scenes[settings['Scene_Name']].cycles.time_limit = settings['Time_Limit']
            bpy.data.scenes[settings['Scene_Name']].cycles.use_adaptive_sampling = settings['Use_Adaptive']

def render_post(scene_name):
    #bpy.data.scenes[scene_name].use_nodes = False
    
    try:        
        revert_all_scenes_back_to_the_users_original_render_settings()
        
    except Exception as e:

        print(str(e))
        from ...utils.debugging_helpers import output_stack_trace
        output_stack_trace()
        return False
    return True
  
    


