import bpy
import queue
from ..variables import global_vars
from .. operators.dont_register import report_custom_error


class SnapshotQueue(queue.Queue):
    def snapshot(self):
        with self.mutex:
            return list(self.queue)
        
execution_queue = SnapshotQueue()


def get_queued_items():
    return execution_queue.snapshot()

# queued functions will be executed when blender returns from the main thread after rendering
def add_function_to_queue(function_and_args):
    """pass in a list containing [function,arg,arg,arg,etc]"""
    execution_queue.put(function_and_args)
    print('queue initialised')

def execute_queued_functions():
    
    if execution_queue.empty():
        return None
    
    if bpy.context.screen.is_animation_playing:    
        return 0.0001
    if bpy.app.version >= (3,3,0):
        if bpy.app.is_job_running('RENDER'):
            return 0.0001
        if bpy.app.is_job_running('COMPOSITE'):
            return 0.0001
    

    while not execution_queue.empty():
        #print('doing the queued function')
        items = execution_queue.get()        
        #print(execution_queue)
        func = items[0]
        args = items[1:]
        if not global_vars.reset_tree_attempt_count:
            global_vars.reset_tree_attempt_count = 1
        else:
            global_vars.reset_tree_attempt_count += 1
        
        success = func(*args)
        
        if success:
            print('cancelled = ',global_vars.cancelled)
            if global_vars.cancelled is None:
                bpy.ops.bugtest.repeat_render()
            #bpy.ops.render.render("INVOKE_DEFAULT",animation=False,use_viewport=True,write_still=False)
            return None
       

        elif global_vars.reset_tree_attempt_count < 11:
            #try to reset again
            print('retry tree revert',global_vars.reset_tree_attempt_count)
            add_function_to_queue([func,args[0]])
            return 0.0001
        else:
            print('an error has occurred.  Please check the console for details.')
            report_custom_error.notify_user(args[0],['an error has occurred.  Please check the console for details.'])           
            
            return None



def register_timer():    
    bpy.app.timers.register(execute_queued_functions)


def unregister_timer():
    bpy.app.timers.unregister(execute_queued_functions)

